from image_builder import repo_create

import stub_koji
from mock import Mock, call
import os
import urllib


class TestableRepoCreate(repo_create.RepoCreate):

    def __init__(self, repodir, data):
        self.data = data
        #self.repodir = repodir
        super(self.__class__, self).__init__(repodir)

    def get_koji_connection(self, url='http://localhost/kojihub'):
        return stub_koji.StubKoji(self.data)

def make_nvr(name, version, release):
    return "%s-%s.%s" % (name, version, release)

default_buildid = 12345
default_name = 'foo'
default_version = '1.2'
default_release = '3.fc99'
default_nvr = make_nvr(default_name, default_version, default_release)

def generate_build_dict(build_id=default_buildid, name=default_name,
                        version=default_version, release=default_release):
    build = {
            'id': build_id,
            'package_name': name,
            'creation_time': '2012-01-15 23:13:23.127916',
            'epoch': None,
            'nvr': make_nvr(name, version, release),
            'name': name,
            'completion_time': '2012-01-15 23:17:42.088126',
            'version': version,
            'release': release
            }

    return build

def generate_rpm_data(build_id=default_buildid, name=default_name,
                        version=default_version, release=default_release,
                        arch='noarch'):
    rpm = {
            'build_id': build_id,
            'nvr': make_nvr(name, version, release),
            'name': name,
            'epoch': None,
            'version': version,
            'arch': arch,
            'release': release
            }
    return rpm



class TestRepoCreate():

    def test_get_build(self):
        ref_build = generate_build_dict()
        ref_data = {'builds' : [ref_build]}

        test_create = TestableRepoCreate('.', ref_data)

        test_build = test_create.get_build(default_nvr)

        assert test_build['id'] == ref_build['id']

    def test_get_rpms_one_noarch(self):
        ref_rpms = [
                    generate_rpm_data(),
                    generate_rpm_data(arch='src'),
                    generate_rpm_data(arch='x86_64')
                   ]

        ref_data = {'rpms' : ref_rpms}

        test_create = TestableRepoCreate('.', ref_data)

        # this should only find the noarch rpm
        test_rpms = test_create.get_rpms(default_buildid, arches=['noarch'])

        assert len(test_rpms) == 1
        assert test_rpms[0]['arch'] == 'noarch'

    def test_get_rpms_two_x64(self):
        ref_rpms = [
                    generate_rpm_data(),
                    generate_rpm_data(arch='src'),
                    generate_rpm_data(arch='x86_64'),
                    generate_rpm_data(arch='x86_64', name='bar')
                   ]

        ref_data = {'rpms' : ref_rpms}

        test_create = TestableRepoCreate('.', ref_data)

        # this should find 2 x86_64 rpms
        test_rpms = test_create.get_rpms(default_buildid, arches=['x86_64'])

        assert len(test_rpms) == 2
        assert test_rpms[0]['arch'] == 'x86_64'
        assert test_rpms[1]['arch'] == 'x86_64'

    def test_get_rpm_urls(self):

        ref_rpms = [
                    generate_rpm_data(),
                    generate_rpm_data(arch='src'),
                    generate_rpm_data(arch='x86_64'),
                    generate_rpm_data(arch='x86_64', name='bar')
                   ]

        ref_build = generate_build_dict()
        ref_data = {'builds' : [ref_build], 'rpms' : ref_rpms}

        test_create = TestableRepoCreate('.', ref_data)

        test_urls = test_create.get_rpm_urls(default_nvr)

        assert len(test_urls) == 3

    def test_prep_repodir_noexist(self, monkeypatch):
        ref_path = '/srv/repo/testrepo'
        mock_path = Mock()
        mock_path.return_value = False

        mock_dir = Mock()
        mock_dir.return_value = True

        test_create = TestableRepoCreate(ref_path, {})

        monkeypatch.setattr(os.path, 'exists', mock_path)
        monkeypatch.setattr(os, 'makedirs', mock_dir)

        test_create.prep_repo_dir()

        assert mock_path.call_args_list == [call(ref_path)]
        assert mock_dir.call_args_list == [call(ref_path)]

    def test_download_rpms(self, monkeypatch):
        ref_rpms = [
                    generate_rpm_data(),
                    generate_rpm_data(arch='x86_64', name='bar')
                   ]
        for rpm in ref_rpms:
            rpm['url'] = 'http://localhost/rpms/%s-%s-%s.%s.rpm' % (rpm['name'],
                                rpm['version'], rpm['release'], rpm['arch'])

        ref_rpm_nvrs = [default_nvr]
        ref_arches = ['x86_64','noarch']
        ref_repodir = '/srv/repos/test/'

        mock_urllib = Mock()
        mock_urllib.return_value = ('downloaded_rpm', 'HTTPMessage')

        mock_rpm_urls = Mock()
        mock_rpm_urls.return_value = ref_rpms

        test_create = TestableRepoCreate(ref_repodir, {})

        monkeypatch.setattr(urllib, 'urlretrieve', mock_urllib)
        monkeypatch.setattr(test_create, 'get_rpm_urls', mock_rpm_urls)

        test_create.download_packages(ref_rpm_nvrs, ref_arches)

        assert mock_urllib.call_args_list == [call(r['url'], dest=ref_repodir) for r in ref_rpms]
        assert mock_rpm_urls.call_args_list == [call(default_nvr, ['x86_64', 'noarch'])]




#    def test_get_build_no_connection(self):
#        ref_data = {'error' : 'socket_error'}
#        ref_nvr = self.ref_build['nvr']
#
#        test_create = TestableRepoCreate('.', ref_data)
#
#        test_build = test_create.get_build(ref_nvr)

